@@ -38,6 +38,7 @@
 #include <eel/eel-gtk-macros.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkdialog.h>
+#include <gtk/gtkbindings.h>
 #include <gtk/gtkcellrendererpixbuf.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkentry.h>
@@ -859,9 +860,9 @@ cell_renderer_edited (GtkCellRendererText *cell,
 	/* Only rename if name actually changed */
 	if (strcmp (new_text, view->details->original_name) != 0) {
 		fm_rename_file (file, new_text);
+		g_free (view->details->original_name);
+		view->details->original_name = g_strdup (new_text);
 	}
-	g_free (view->details->original_name);
-	view->details->original_name = NULL;
 	
 	nautilus_file_unref (file);
 
@@ -1004,6 +1005,7 @@ create_and_set_up_tree_view (FMListView *view)
 	GtkCellRenderer *cell;
 	GtkTreeViewColumn *column;
 	GtkTargetEntry *drag_types;
+	GtkBindingSet *binding_set;
 	AtkObject *atk_obj;
 	int num_drag_types;	
 	GList *nautilus_columns;
@@ -1015,6 +1017,10 @@ create_and_set_up_tree_view (FMListView *view)
 							(GDestroyNotify)g_free,
 							NULL);
 
+	/* Don't handle backspace key. It's used to open the parent folder. */
+	binding_set = gtk_binding_set_by_class (GTK_WIDGET_GET_CLASS (view->details->tree_view));
+	gtk_binding_entry_clear (binding_set, GDK_BackSpace, 0);
+
 	fm_list_model_get_drag_types (&drag_types, &num_drag_types);
 	
 	view->details->drag_dest = 
@@ -1917,6 +1923,10 @@ fm_list_view_start_renaming_file (FMDirectoryView *view, NautilusFile *file)
 				  TRUE);
 
 	entry = GTK_ENTRY (list_view->details->file_name_column->editable_widget);
+
+	/* Free a previously allocated original_name */
+	g_free (list_view->details->original_name);
+
 	list_view->details->original_name = g_strdup (gtk_entry_get_text (entry));
 	eel_filename_get_rename_region (list_view->details->original_name,
 					&start_offset, &end_offset);
