@@ -29,13 +29,16 @@
 #include "nautilus-application.h"
 
 
-#include "file-manager/fm-bonobo-provider.h"
 #include "file-manager/fm-ditem-page.h"
 #include "file-manager/fm-desktop-icon-view.h"
 #include "file-manager/fm-icon-view.h"
 #include "file-manager/fm-list-view.h"
-#include "file-manager/fm-search-list-view.h"
 #include "file-manager/fm-tree-view.h"
+#include "nautilus-information-panel.h"
+#include "nautilus-history-sidebar.h"
+#include "nautilus-notes-viewer.h"
+#include "nautilus-emblem-sidebar.h"
+#include "nautilus-image-properties-page.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -76,11 +79,12 @@
 #include <libnautilus-private/nautilus-metafile-factory.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-sound.h>
-#include <libnautilus-private/nautilus-bonobo-extensions.h>
 #include <libnautilus-private/nautilus-undo-manager.h>
 #include <libnautilus-private/nautilus-desktop-link-monitor.h>
 #include <libnautilus-private/nautilus-directory-private.h>
 #include <bonobo-activation/bonobo-activation.h>
+#define SN_API_NOT_YET_FROZEN Yes_i_know_DO_IT
+#include <libsn/sn-launchee.h>
 
 /* Needed for the is_kdesktop_present check */
 #include <gdk/gdkx.h>
@@ -121,28 +125,13 @@ create_object (PortableServer_Servant servant,
 	       CORBA_Environment *ev)
 {
 	BonoboObject *object;
-	FMDirectoryView *directory_view;
 	NautilusApplication *application;
 
-	if (strcmp (iid, NAUTILUS_ICON_VIEW_IID) == 0) {
-		directory_view = FM_DIRECTORY_VIEW (g_object_new (fm_icon_view_get_type (), NULL));
-		object = BONOBO_OBJECT (fm_directory_view_get_nautilus_view (directory_view));
-	} else if (strcmp (iid, NAUTILUS_DESKTOP_ICON_VIEW_IID) == 0) {
-		directory_view = FM_DIRECTORY_VIEW (g_object_new (fm_desktop_icon_view_get_type (), NULL));
-		object = BONOBO_OBJECT (fm_directory_view_get_nautilus_view (directory_view));
-	} else if (strcmp (iid, NAUTILUS_LIST_VIEW_IID) == 0) {
-		directory_view = FM_DIRECTORY_VIEW (g_object_new (fm_list_view_get_type (), NULL));
-		object = BONOBO_OBJECT (fm_directory_view_get_nautilus_view (directory_view));
-	} else if (strcmp (iid, SEARCH_LIST_VIEW_IID) == 0) {
-		directory_view = FM_DIRECTORY_VIEW (g_object_new (fm_search_list_view_get_type (), NULL));
-		object = BONOBO_OBJECT (fm_directory_view_get_nautilus_view (directory_view));
-	} else if (strcmp (iid, SHELL_IID) == 0) {
+	if (strcmp (iid, SHELL_IID) == 0) {
 		application = NAUTILUS_APPLICATION (bonobo_object_from_servant (servant));
 		object = BONOBO_OBJECT (nautilus_shell_new (application));
 	} else if (strcmp (iid, METAFILE_FACTORY_IID) == 0) {
 		object = BONOBO_OBJECT (nautilus_metafile_factory_get_instance ());
-	} else if (strcmp (iid, TREE_VIEW_IID) == 0) {
-		object = BONOBO_OBJECT (g_object_new (fm_tree_view_get_type (), NULL));
 	} else {
 		object = CORBA_OBJECT_NIL;
 	}
@@ -162,12 +151,6 @@ nautilus_application_get_spatial_window_list (void)
 	return nautilus_application_spatial_window_list;
 }
 
-static CORBA_Object
-create_object_shortcut (const char *iid,
-			gpointer callback_data)
-{
-	return create_object (BONOBO_OBJREF (callback_data), iid, NULL);
-}
 
 static void
 nautilus_application_instance_init (NautilusApplication *application)
@@ -187,11 +170,20 @@ nautilus_application_instance_init (NautilusApplication *application)
 	g_signal_connect_object (gnome_vfs_get_volume_monitor (), "volume_mounted",
 				 G_CALLBACK (volume_mounted_callback), application, 0);
 
-	nautilus_bonobo_register_activation_shortcut (NAUTILUS_ICON_VIEW_IID, create_object_shortcut, application);
-	nautilus_bonobo_register_activation_shortcut (NAUTILUS_DESKTOP_ICON_VIEW_IID, create_object_shortcut, application);
-	nautilus_bonobo_register_activation_shortcut (NAUTILUS_LIST_VIEW_IID, create_object_shortcut, application);
-	nautilus_bonobo_register_activation_shortcut (SEARCH_LIST_VIEW_IID, create_object_shortcut, application);
-	nautilus_bonobo_register_activation_shortcut (TREE_VIEW_IID, create_object_shortcut, application);
+	/* register views */
+	fm_icon_view_register ();
+	fm_desktop_icon_view_register ();
+	fm_list_view_register ();
+
+	/* register sidebars */
+	nautilus_information_panel_register ();
+	fm_tree_view_register ();
+	nautilus_history_sidebar_register ();
+	nautilus_notes_viewer_register (); /* also property page */
+	nautilus_emblem_sidebar_register ();
+
+	/* register property pages */
+	nautilus_image_properties_page_register ();
 }
 
 NautilusApplication *
@@ -215,15 +207,9 @@ nautilus_application_destroy (BonoboObject *object)
 
 	application = NAUTILUS_APPLICATION (object);
 
-	nautilus_bonobo_unregister_activation_shortcut (NAUTILUS_ICON_VIEW_IID);
-	nautilus_bonobo_unregister_activation_shortcut (NAUTILUS_DESKTOP_ICON_VIEW_IID);
-	nautilus_bonobo_unregister_activation_shortcut (NAUTILUS_LIST_VIEW_IID);
-	nautilus_bonobo_unregister_activation_shortcut (SEARCH_LIST_VIEW_IID);
-	nautilus_bonobo_unregister_activation_shortcut (TREE_VIEW_IID);
-	
 	nautilus_bookmarks_exiting ();
 	
-	bonobo_object_unref (application->undo_manager);
+	g_object_unref (application->undo_manager);
 
 	EEL_CALL_PARENT (BONOBO_OBJECT_CLASS, destroy, (object));
 }
@@ -373,7 +359,6 @@ finish_startup (NautilusApplication *application)
 	/* initialize nautilus modules */
 	nautilus_module_init ();
 
-	nautilus_module_add_type (FM_TYPE_BONOBO_PROVIDER);
 	nautilus_module_add_type (FM_TYPE_DITEM_PAGE);
 	
 	/* initialize the sound machinery */
@@ -444,6 +429,23 @@ initialize_kde_trash_hack (void)
 	g_free (desktop_dir);
 }
 
+
+static Bonobo_RegistrationResult
+nautilus_bonobo_activation_register_for_display (const char    *iid,
+						 Bonobo_Unknown ref)
+{
+	const char *display_name;
+	GSList *reg_env ;
+	Bonobo_RegistrationResult result;
+	
+	display_name = gdk_display_get_name (gdk_display_get_default());
+	reg_env = bonobo_activation_registration_env_set (NULL,
+							  "DISPLAY", display_name);
+	result = bonobo_activation_register_active_server (iid, ref, reg_env);
+	bonobo_activation_registration_env_free (reg_env);
+	return result;
+}
+
 void
 nautilus_application_startup (NautilusApplication *application,
 			      gboolean kill_shell,
@@ -452,6 +454,7 @@ nautilus_application_startup (NautilusApplication *application,
 			      gboolean no_desktop,
 			      gboolean do_first_time_druid_check,
 			      gboolean browser_window,
+			      const char *startup_id,
 			      const char *geometry,
 			      const char *urls[])
 {
@@ -461,6 +464,7 @@ nautilus_application_startup (NautilusApplication *application,
 	const char *message, *detailed_message;
 	GtkDialog *dialog;
 	Nautilus_URIList *url_list;
+	const CORBA_char *corba_startup_id;
 	const CORBA_char *corba_geometry;
 	int num_failures;
 
@@ -610,15 +614,16 @@ nautilus_application_startup (NautilusApplication *application,
 
 		/* CORBA C mapping doesn't allow NULL to be passed
 		   for string parameters */
-		corba_geometry = (geometry != NULL) ? geometry : "";
+		corba_geometry   = (geometry   != NULL) ? geometry   : "";
+		corba_startup_id = (startup_id != NULL) ? startup_id : "";
 
 	  	/* Create the other windows. */
 		if (urls != NULL) {
 			url_list = nautilus_make_uri_list_from_shell_strv (urls);
-			Nautilus_Shell_open_windows (shell, url_list, corba_geometry, browser_window, &ev);
+			Nautilus_Shell_open_windows (shell, url_list, corba_startup_id, corba_geometry, browser_window, &ev);
 			CORBA_free (url_list);
 		} else if (!no_default_window) {
-			Nautilus_Shell_open_default_window (shell, corba_geometry, browser_window, &ev);
+			Nautilus_Shell_open_default_window (shell, corba_startup_id, corba_geometry, browser_window, &ev);
 		}
 		
 		/* Add ourselves to the session */
@@ -822,6 +827,9 @@ find_parent_spatial_window (NautilusSpatialWindow *window)
 	char *desktop_directory;
 
 	location = nautilus_window_get_location (NAUTILUS_WINDOW (window));
+	if (location == NULL) {
+		return NULL;
+	}
 	file = nautilus_file_get (location);
 	g_free (location);
 
@@ -923,6 +931,7 @@ nautilus_window_delete_event_callback (GtkWidget *widget,
 static NautilusWindow *
 create_window (NautilusApplication *application,
 	       GType window_type,
+	       const char *startup_id,
 	       GdkScreen *screen)
 {
 	NautilusWindow *window;
@@ -931,9 +940,10 @@ create_window (NautilusApplication *application,
 	
 	window = NAUTILUS_WINDOW (gtk_widget_new (window_type,
 						  "app", application,
-						  "app_id", "nautilus",
 						  "screen", screen,
 						  NULL));
+	/* Must be called after construction finished */
+	nautilus_window_constructed (window);
 
 	g_signal_connect_data (window, "delete_event",
 			       G_CALLBACK (nautilus_window_delete_event_callback), NULL, NULL,
@@ -962,22 +972,100 @@ spatial_window_destroyed_callback (void *user_data, GObject *window)
 NautilusWindow *
 nautilus_application_present_spatial_window (NautilusApplication *application,
 					     NautilusWindow      *requesting_window,
+					     const char          *startup_id,
 					     const char          *location,
 					     GdkScreen           *screen)
 {
 	return nautilus_application_present_spatial_window_with_selection (application,
 									   requesting_window,
+									   startup_id,
 									   location,
 									   NULL,
 									   screen);
 }
 
+static void
+sn_error_trap_push (SnDisplay *display,
+                    Display   *xdisplay)
+{
+	gdk_error_trap_push ();
+}
+
+static void
+sn_error_trap_pop (SnDisplay *display,
+                   Display   *xdisplay)
+{
+	gdk_error_trap_pop ();
+}
+
+static void
+end_startup_notification (GtkWidget  *widget, 
+			  const char *startup_id)
+{
+	SnDisplay *sn_display;
+	SnLauncheeContext *context;
+	GdkDisplay *display;
+	GdkScreen  *screen;
+
+	if (startup_id == NULL) {
+		return;
+	}
+  
+	if (!GTK_WIDGET_REALIZED (widget)) {
+		gtk_widget_realize (widget);
+	}
+  
+	context = NULL;
+	sn_display = NULL;
+
+	/* Set up window for launch notification */
+	/* FIXME In principle all transient children of this
+	 * window should get the same startup_id
+	 */
+
+	screen = gtk_widget_get_screen (widget);
+	display = gdk_screen_get_display (screen);
+      
+	sn_display = sn_display_new (gdk_x11_display_get_xdisplay (display),
+				     sn_error_trap_push,
+				     sn_error_trap_pop);
+      
+	context = sn_launchee_context_new (sn_display,
+					   gdk_screen_get_number (screen),
+					   startup_id);
+
+	sn_launchee_context_setup_window (context,
+					  GDK_WINDOW_XWINDOW (widget->window));
+
+	/* Now, set the _NET_WM_USER_TIME for the new window to the timestamp
+	 * that caused the window to be launched.
+	 */
+	if (sn_launchee_context_get_id_has_timestamp (context)) {
+		gulong startup_id_timestamp;
+		startup_id_timestamp = sn_launchee_context_get_timestamp (context);
+		gdk_x11_window_set_user_time (widget->window, startup_id_timestamp);
+	} else {
+		/* Comment this out for now, as it warns way to often.
+		 * like when launching nautilus from the terminal.
+		 
+		g_warning ("Launched by a non-compliant or obsolete startup "
+			   "notification launcher.  Focus-stealing-prevention "
+			   "may fail.\n");
+		*/
+	}
+  
+	sn_launchee_context_complete (context);
+	sn_launchee_context_unref (context);
+	sn_display_unref (sn_display);
+}
+
 NautilusWindow *
 nautilus_application_present_spatial_window_with_selection (NautilusApplication *application,
-					     NautilusWindow      *requesting_window,
-					     const char          *location,
-					     GList		 *new_selection,
-					     GdkScreen           *screen)
+							    NautilusWindow      *requesting_window,
+							    const char          *startup_id,
+							    const char          *location,
+							    GList		*new_selection,
+							    GdkScreen           *screen)
 {
 	NautilusWindow *window;
 	GList *l;
@@ -991,21 +1079,26 @@ nautilus_application_present_spatial_window_with_selection (NautilusApplication
                	     
 		existing_window = NAUTILUS_WINDOW (l->data);
 		existing_location = existing_window->details->pending_location;
-                
+		
 		if (existing_location == NULL) {
 			existing_location = existing_window->details->location;
 		}
 
 		if (eel_uris_match (existing_location, location)) {
+			end_startup_notification (GTK_WIDGET (existing_window),
+						  startup_id);
+
 			gtk_window_present (GTK_WINDOW (existing_window));
 			if (new_selection) {
-				nautilus_view_frame_selection_changed (existing_window->content_view, new_selection);
+				nautilus_view_set_selection (existing_window->content_view, new_selection);
 			}
 			return existing_window;
 		}
 	}
 
-	window = create_window (application, NAUTILUS_TYPE_SPATIAL_WINDOW, screen);
+	window = create_window (application, NAUTILUS_TYPE_SPATIAL_WINDOW, startup_id, screen);
+	end_startup_notification (GTK_WIDGET (window),
+				  startup_id);
 	if (requesting_window) {
 		/* Center the window over the requesting window by default */
 		int orig_x, orig_y, orig_width, orig_height;
@@ -1040,13 +1133,16 @@ nautilus_application_present_spatial_window_with_selection (NautilusApplication
 
 NautilusWindow *
 nautilus_application_create_navigation_window (NautilusApplication *application,
+					       const char          *startup_id,
 					       GdkScreen           *screen)
 {
 	NautilusWindow *window;
 
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
 	
-	window = create_window (application, NAUTILUS_TYPE_NAVIGATION_WINDOW, screen);
+	window = create_window (application, NAUTILUS_TYPE_NAVIGATION_WINDOW, startup_id, screen);
+	end_startup_notification (GTK_WIDGET (window),
+				  startup_id);
 
 	return window;
 }
